import click
import random

from dapodik import __semester__

from dapodix import ClickContext, ContextObject
from dapodix.utils import parse_range

from .excel import (
    DATA_INDIVIDU,
    DATA_REGISTRASI,
    DATA_AYAH,
    DATA_IBU,
    DATA_LONGITUDINAL,
    ALL_DATA_INDIVIDU,
)
from .eksport import EksporPesertaDidikCommand
from .random_longitudinal import RandomLongitudinal
from .registrasi import RegistrasiPesertaDidikCommand


@click.group(name="peserta_didik", invoke_without_command=True)
@click.option("--email", required=True, help="Email dapodik")
@click.option(
    "--password",
    required=True,
    prompt=True,
    hide_input=True,
    confirmation_prompt=True,
    help="Password dapodik",
)
@click.option("--server", default="http://localhost:5774/", help="URL aplikasi dapodik")
@click.option("--semester", default=__semester__, help="Semester id")
@click.pass_context
def peserta_didik(
    ctx: ClickContext, email: str, password: str, server: str, semester: str
):
    ctx.ensure_object(ContextObject)
    ctx.obj.username = email
    ctx.obj.password = password
    ctx.obj.server = server
    ctx.obj.semester = semester
    if ctx.invoked_subcommand is None:
        dapodik = ctx.obj.dapodik
        sekolah = dapodik.sekolah()
        click.echo("Daftar Peserta didik")
        for pd in dapodik.peserta_didik(sekolah_id=sekolah.sekolah_id):
            click.echo(str(pd))


@peserta_didik.command()
@click.option("--sheet", default="Peserta Didik", help="Nama sheet dalam file excel")
@click.option(
    "--header/--no-header",
    default=True,
    help="Tambah header setiap kolom",
)
@click.option(
    "--longitudinal/--no-longitudinal",
    default=True,
    help="Proses data periodik",
)
@click.argument("filepath", type=click.Path(), required=True)
@click.pass_context
def ekspor(
    ctx: ClickContext,
    sheet: str,
    header: bool,
    filepath: str,
    longitudinal: bool,
):
    return EksporPesertaDidikCommand(
        dapodik=ctx.obj.dapodik,
        filepath=filepath,
        sheet=sheet,
        header=header,
        skip_longitudinal=not longitudinal,
    )


@peserta_didik.command()
@click.option("--sheet", default="Peserta Didik", help="Nama sheet dalam file excel")
@click.option(
    "--range",
    required=True,
    help="Baris data yang akan di masukkan misal 1-10",
)
@click.option(
    "--registrasi/--no-registrasi",
    default=True,
    help="Proses registrasi",
)
@click.option(
    "--longitudinal/--no-longitudinal",
    default=True,
    help="Proses data periodik",
)
@click.argument("filepath", type=click.Path(exists=True), required=True)
@click.pass_context
def registrasi(
    ctx: ClickContext,
    filepath: str,
    sheet: str,
    range: str,
    registrasi: bool,
    longitudinal: bool,
):
    return RegistrasiPesertaDidikCommand(
        dapodik=ctx.obj.dapodik,
        filepath=filepath,
        sheet=sheet,
        rows=parse_range(range),
        skip_registrasi=not registrasi,
        skip_longitudinal=not longitudinal,
    )


@peserta_didik.command()
@click.option("--tinggi-badan", required=True)
@click.option("--berat-badan", required=True)
@click.option("--lingkar-kepala", required=True)
@click.option("--jarak-rumah", required=True)
@click.option("--jarak-ke-waktu", type=int, required=True)
@click.pass_context
def random_longitudinal(
    ctx: ClickContext,
    tinggi_badan: str,
    berat_badan: str,
    lingkar_kepala: str,
    jarak_rumah: str,
    jarak_ke_waktu: int,
):
    return RandomLongitudinal(
        dapodik=ctx.obj.dapodik,
        tinggi=lambda: random.choice(parse_range(tinggi_badan)),
        berat=lambda: random.choice(parse_range(berat_badan)),
        lingkar_kepala=lambda: random.choice(parse_range(lingkar_kepala)),
        jarak_rumah=lambda: random.choice(parse_range(jarak_rumah)),
        jarak_waktu=jarak_ke_waktu,
    )



import tkinter as tk
from dapodik import __semester__, Dapodik


class LoginFrame(tk.Frame):
    def __init__(self, master=None, on_login=None, cnf=None, **kw):
        super(LoginFrame, self).__init__(master, cnf=cnf if cnf else {}, **kw)
        on_login = on_login if callable(on_login) else self.login
        # Email Form
        self.emailLabel = tk.Label(self, text="Email")
        self.emailLabel.grid(row=0, column=0)
        self.email = tk.StringVar(self)
        self.emailEntry = tk.Entry(self, textvariable=self.email)
        self.emailEntry.grid(row=0, column=1)
        # Password Form
        self.passwordLabel = tk.Label(self, text="Password")
        self.passwordLabel.grid(row=1, column=0)
        self.password = tk.StringVar(self)
        self.passwordEntry = tk.Entry(self, textvariable=self.password, show="*")
        self.passwordEntry.grid(row=1, column=1)
        # Semester Form
        self.semesterLabel = tk.Label(self, text="Semester")
        self.semesterLabel.grid(row=2, column=0)
        self.semester = tk.StringVar(self, value=__semester__)
        self.semesterEntry = tk.Entry(self, textvariable=self.semester)
        self.semesterEntry.grid(row=2, column=1)
        # Server Form
        self.serverLabel = tk.Label(self, text="Server")
        self.serverLabel.grid(row=3, column=0)
        self.server = tk.StringVar(self, value="http://localhost:5774/")
        self.serverEntry = tk.Entry(self, textvariable=self.server)
        self.serverEntry.grid(row=3, column=1)
        # Button
        self.loginButton = tk.Button(self, text="Masuk", command=on_login)
        self.loginButton.grid(row=5, column=0)

    def login(self):
        email = self.emailEntry.get()
        password = self.passwordEntry.get()
        print(f"{email} {password}")

    def dapodik(self) -> Dapodik:
        return Dapodik(
            username=self.emailEntry.get(),
            password=self.passwordEntry.get(),
            semester_id=self.semesterEntry.get(),
            server=self.serverEntry.get(),
        )



import click
import tkinter as tk

from dapodik import __dapodik_version__, __semester__, __tahun_ajaran__

from . import __version__
from .peserta_didik import peserta_didik
from .gui import MainApplication


@click.group("dapodix")
def main():
    pass


@main.command("gui")
def gui():
    root = tk.Tk()
    MainApplication(root).pack(side="top", fill="both", expand=True)
    root.mainloop()


main.add_command(peserta_didik, "peserta-didik")


@main.command()
def version():
    click.echo(f"Dapodix versi {__version__}")
    click.echo(f"Untuk dapodik {__dapodik_version__}")
    click.echo(f"Semester : {__semester__}")
    click.echo(f"Tahun ajaran : {__tahun_ajaran__}")


if __name__ == "__main__":
    main()



from cachetools import cachedmethod
from operator import attrgetter
from typing import Any, Dict, List, Optional, Union

from dapodik import Dapodik, __semester__
from dapodik.customrest import Wilayah
from dapodik.peserta_didik import (
    CreatePesertaDidik,
    PesertaDidik,
    PesertaDidikLongitudinal,
    RegistrasiPesertaDidik,
)
from dapodik.rest import (
    Agama,
    JenisCita,
    JenisHobby,
    JenisPendaftaran,
    JenjangPendidikan,
    Pekerjaan,
    Penghasilan,
)
from dapodik.sekolah import Sekolah

from dapodix.utils import get_data_excel

from . import DATA_INDIVIDU, DATA_REGISTRASI, DATA_LONGITUDINAL, DATA_AYAH, DATA_IBU


class RegistrasiPesertaDidikCommand:
    JARAK_KE_MENIT = 5
    nisn = ""
    kebutuhan_khusus_id = 0
    lintang = 0
    bujur = 0
    jenis_tinggal_id = 1
    alat_transportasi_id = 1
    anak_keberapa = 1
    penerima_kps = 0
    penerima_kip = 0
    layak_pip = 0
    id_layak_pip = None
    kewarganegaraan = "ID"
    no_kks = ""
    no_kps = ""
    no_kip = ""
    nm_kip = 0
    # Data Ayah Kandung
    kebutuhan_khusus_id_ayah = 0
    # Data Ibu Kandung
    kebutuhan_khusus_id_ibu = 0
    # Data Wali
    nama_wali = ""
    nik_wali = ""
    tahun_lahir_wali = ""
    jenjang_pendidikan_wali = 0
    pekerjaan_id_wali = 0
    penghasilan_id_wali = 0
    # Kontak
    nomor_telepon_rumah = ""
    nomor_telepon_seluler = ""
    email = ""

    def __init__(
        self,
        dapodik: Dapodik,
        filepath: str,
        sheet: str,
        rows: List[int],
        skip_registrasi: bool = False,
        skip_longitudinal: bool = False,
    ):
        self.dapodik = dapodik
        self.filepath = filepath
        self.sheet = sheet
        self.rows = rows
        self.skip_registrasi = skip_registrasi
        self.skip_longitudinal = skip_longitudinal
        self.sekolah: Sekolah = self.dapodik.sekolah()
        # Individu
        self.agama_cache: Dict[str, int] = dict()
        self.kode_wilayah_cache: Dict[str, str] = dict()
        # Orang tua
        self.penghasilan_cache: Dict[tuple, int] = dict()
        self.jenjang_pendidikan_cache: Dict[str, int] = dict()
        self.pekerjaan_cache: Dict[str, int] = dict()
        # Registrasi
        self.cita_cache: Dict[str, int] = dict()
        self.hobby_cache: Dict[str, int] = dict()
        self.pendaftaran_cache: Dict[str, int] = dict()
        # INDIVIDU
        self.AGAMA: List[Agama] = self.dapodik.agama()
        # ORANG TUA
        self.JENJANG_PENDIDIKAN: List[
            JenjangPendidikan
        ] = self.dapodik.jenjang_pendidikan()
        self.PEKERJAAN: List[Pekerjaan] = self.dapodik.pekerjaan()
        self.PENGHASILAN: List[Penghasilan] = self.dapodik.penghasilan()
        # REGISTRASI
        self.CITA: List[JenisCita] = self.dapodik.jenis_cita()
        self.HOBBY: List[JenisHobby] = self.dapodik.jenis_hobby()
        self.PENDAFTARAN: List[JenisPendaftaran] = self.dapodik.jenis_pendaftaran()

        self.start()

    def start(self) -> Dict[int, PesertaDidik]:
        raw_data_individu = get_data_excel(
            self.filepath,
            self.sheet,
            self.rows,
            DATA_INDIVIDU,
        )
        raw_data_ayah = get_data_excel(
            self.filepath,
            self.sheet,
            self.rows,
            DATA_AYAH,
        )
        raw_data_ibu = get_data_excel(
            self.filepath,
            self.sheet,
            self.rows,
            DATA_IBU,
        )
        if self.skip_registrasi:
            raw_data_registrasi: Dict[int, Dict[str, Any]] = dict()
        else:
            raw_data_registrasi = get_data_excel(
                self.filepath,
                self.sheet,
                self.rows,
                DATA_REGISTRASI,
            )
        if self.skip_longitudinal:
            raw_data_longitudinal: Dict[int, Dict[str, Any]] = dict()
        else:
            raw_data_longitudinal = get_data_excel(
                self.filepath,
                self.sheet,
                self.rows,
                DATA_LONGITUDINAL,
            )
        peserta_didik_baru: Dict[int, PesertaDidik] = dict()
        for index, data_individu in raw_data_individu.items():
            # Individu
            data_ayah = raw_data_ayah[index]
            data_ibu = raw_data_ibu[index]
            data = self.get_default()
            data.update(self.transform_data_individu(**data_individu))
            data.update(self.transform_data_ayah(**data_ayah))
            data.update(self.transform_data_ibu(**data_ibu))
            peserta_didik = self.tambah_peserta_didik(data)
            peserta_didik._dapodik = self.dapodik
            # Registrasi
            if not self.skip_registrasi:
                data_registrasi = raw_data_registrasi[index]
                data = self.get_default_registrasi()
                data.update(self.transform_data_registrasi(**data_registrasi))
                peserta_didik = self.registrasi_peserta_didik(data, peserta_didik)
            # Longitudinal
            if not self.skip_longitudinal:
                data_longitudinal = raw_data_longitudinal[index]
                data = self.get_default_longitudinal()
                data.update(self.transform_data_longitudinal(**data_longitudinal))
                peserta_didik = self.longitudinal_peserta_didik(data, peserta_didik)
            # Collect
            peserta_didik_baru[index] = peserta_didik
        return peserta_didik_baru

    def tambah_peserta_didik(self, data: dict) -> PesertaDidik:
        pdb = CreatePesertaDidik(
            kode_wilayah_str=data["kode_wilayah"],
            **data,
        )
        return pdb.save(self.dapodik)

    def longitudinal_peserta_didik(
        self, data: dict, peserta_didik: PesertaDidik
    ) -> PesertaDidik:
        data["peserta_didik_id"] = peserta_didik.peserta_didik_id
        data["peserta_didik_longitudinal_id"] = "Admin.model.PesertaDidikLongitudinal-5"
        longitudinal = PesertaDidikLongitudinal.Create(**data)
        peserta_didik.create_longitudinal(longitudinal)
        return peserta_didik

    def registrasi_peserta_didik(
        self, data: dict, peserta_didik: PesertaDidik
    ) -> PesertaDidik:
        reg_pd = RegistrasiPesertaDidik(
            peserta_didik_id=peserta_didik.peserta_didik_id,
            sekolah_id=peserta_didik.sekolah_id,
            **data,
        )
        reg_pd.sekolah_id = self.sekolah.sekolah_id
        # peserta_didik._dapodik = self.dapodik
        peserta_didik.registrasi(reg_pd)
        return peserta_didik

    def transform_data_individu(
        self,
        nisn: str,
        nama: str,
        jenis_kelamin: str,
        agama_id: str,
        kode_wilayah: str,
        tempat_lahir: str,
        reg_akta_lahir: str,
        nomor_telepon_seluler: str,
        email: str,
        **kwargs: Any,
    ) -> dict:
        kwargs["nisn"] = nisn if nisn else ""
        kwargs["nama"] = nama.upper()
        jenis_kelamin = jenis_kelamin.upper()
        assert jenis_kelamin in ("L", "P")
        kwargs["jenis_kelamin"] = jenis_kelamin
        kwargs["agama_id"] = self.parse_agama(agama_id)
        kwargs["tempat_lahir"] = tempat_lahir.upper()
        kwargs["kode_wilayah"] = self.find_kode_wilayah(kode_wilayah)
        kwargs["reg_akta_lahir"] = reg_akta_lahir if reg_akta_lahir else ""
        kwargs["nomor_telepon_seluler"] = (
            nomor_telepon_seluler if nomor_telepon_seluler else ""
        )
        kwargs["email"] = email if email else ""
        return kwargs

    def transform_data_registrasi(
        self,
        jenis_pendaftaran_id: str,
        sekolah_asal: str,
        nipd: str,
        id_hobby: str,
        id_cita: str,
        a_pernah_paud: Union[str, int],
        a_pernah_tk: Union[str, int],
        **kwargs: Any,
    ) -> dict:
        kwargs["jenis_pendaftaran_id"] = self.parse_pendaftaran(jenis_pendaftaran_id)
        kwargs["id_hobby"] = self.parse_hobby(id_hobby)
        kwargs["id_cita"] = self.parse_cita(id_cita)
        kwargs["sekolah_asal"] = sekolah_asal if sekolah_asal else ""
        kwargs["nipd"] = nipd if nipd else ""
        if isinstance(a_pernah_paud, int):
            kwargs["a_pernah_paud"] = a_pernah_paud
        else:
            kwargs["a_pernah_paud"] = 1 if a_pernah_paud == "YA" else 0
        if isinstance(a_pernah_tk, int):
            kwargs["a_pernah_tk"] = a_pernah_tk
        else:
            kwargs["a_pernah_tk"] = 1 if a_pernah_tk == "YA" else 0
        return kwargs

    def transform_data_longitudinal(
        self,
        jarak_rumah_ke_sekolah: int,
        menit_tempuh_ke_sekolah: int,
        **kwargs: Any,
    ) -> dict:
        if jarak_rumah_ke_sekolah <= 1:
            kwargs["jarak_rumah_ke_sekolah"] = 1
            kwargs["jarak_rumah_ke_sekolah_km"] = 0
        else:
            kwargs["jarak_rumah_ke_sekolah"] = 2
            kwargs["jarak_rumah_ke_sekolah_km"] = jarak_rumah_ke_sekolah
        if not isinstance(menit_tempuh_ke_sekolah, int):
            menit_tempuh_ke_sekolah = jarak_rumah_ke_sekolah * self.JARAK_KE_MENIT
        kwargs["waktu_tempuh_ke_sekolah"] = menit_tempuh_ke_sekolah // 60
        kwargs["menit_tempuh_ke_sekolah"] = menit_tempuh_ke_sekolah % 60
        return kwargs

    def transform_data_ayah(
        self,
        tahun_lahir_ayah: Union[str, int],
        jenjang_pendidikan_ayah: str,
        pekerjaan_id_ayah: str,
        penghasilan_id_ayah: str,
        **kwargs: Any,
    ) -> dict:
        if isinstance(tahun_lahir_ayah, int):
            kwargs["tahun_lahir_ayah"] = str(tahun_lahir_ayah)
        elif isinstance(tahun_lahir_ayah, str) and tahun_lahir_ayah.isdigit():
            kwargs["tahun_lahir_ayah"] = tahun_lahir_ayah
        else:
            kwargs["tahun_lahir_ayah"] = "19" + kwargs["nik_ayah"][10:12]
        kwargs["jenjang_pendidikan_ayah"] = self.parse_jenjang_pendidikan(
            jenjang_pendidikan_ayah
        )
        kwargs["pekerjaan_id_ayah"] = self.parse_pekerjaan(pekerjaan_id_ayah)
        kwargs["penghasilan_id_ayah"] = self.guest_penghasilan(
            penghasilan_id_ayah, kwargs["pekerjaan_id_ayah"]
        )
        return kwargs

    def transform_data_ibu(
        self,
        tahun_lahir_ibu: Union[str, int],
        jenjang_pendidikan_ibu: str,
        pekerjaan_id_ibu: str,
        penghasilan_id_ibu: str,
        **kwargs: Any,
    ) -> dict:
        if isinstance(tahun_lahir_ibu, int):
            kwargs["tahun_lahir_ibu"] = str(tahun_lahir_ibu)
        elif isinstance(tahun_lahir_ibu, str) and tahun_lahir_ibu.isdigit():
            kwargs["tahun_lahir_ibu"] = tahun_lahir_ibu
        else:
            kwargs["tahun_lahir_ibu"] = "19" + kwargs["nik_ibu"][10:12]
        kwargs["jenjang_pendidikan_ibu"] = self.parse_jenjang_pendidikan(
            jenjang_pendidikan_ibu
        )
        kwargs["pekerjaan_id_ibu"] = self.parse_pekerjaan(pekerjaan_id_ibu)
        kwargs["penghasilan_id_ibu"] = self.guest_penghasilan(
            penghasilan_id_ibu, kwargs["pekerjaan_id_ibu"]
        )
        return kwargs

    @cachedmethod(attrgetter("penghasilan_cache"))
    def guest_penghasilan(self, val: Union[str, int], pekerjaan: int) -> int:
        if isinstance(val, int):
            return val
        penghasilan_id = 99
        if pekerjaan in (90, 98, 1):
            # Tidak bekerja
            pass
        elif val and (isinstance(val, int) or val.isdigit()):
            digit_val = int(val)
            if digit_val <= 0:
                penghasilan_id = 99
            for penghasilan in self.PENGHASILAN:
                if penghasilan.batas_atas < digit_val < penghasilan.batas_bawah:
                    penghasilan_id = penghasilan.penghasilan_id
                elif (
                    penghasilan.batas_bawah == 0 and digit_val > penghasilan.batas_atas
                ):
                    penghasilan_id = penghasilan.penghasilan_id
        elif val:
            for penghasilan in self.PENGHASILAN:
                if val in penghasilan.nama:
                    penghasilan_id = penghasilan.penghasilan_id
        return penghasilan_id if penghasilan_id != 0 else 99

    @cachedmethod(attrgetter("agama_cache"))
    def parse_agama(self, keyword: Union[str, int]) -> int:
        if isinstance(keyword, int):
            return keyword
        for agama in self.AGAMA:
            if keyword in agama.nama:
                return agama.agama_id
        return 1

    @cachedmethod(attrgetter("kode_wilayah_cache"))
    def find_kode_wilayah(self, keyword: str) -> str:
        wilayah: List[Wilayah] = self.dapodik.kecamatan(keyword)
        return wilayah[0].kode_wilayah

    @cachedmethod(attrgetter("jenjang_pendidikan_cache"))
    def parse_jenjang_pendidikan(self, val: Union[str, int]) -> int:
        if isinstance(val, int):
            return val
        for jenjang_pendidikan in self.JENJANG_PENDIDIKAN:
            if val in jenjang_pendidikan.nama:
                return jenjang_pendidikan.jenjang_pendidikan_id
        return 0

    @cachedmethod(attrgetter("pekerjaan_cache"))
    def parse_pekerjaan(self, val: str) -> int:
        if isinstance(val, int):
            return val
        for pekerjaan in self.PEKERJAAN:
            if val in pekerjaan.nama:
                return pekerjaan.pekerjaan_id
        return 1

    @cachedmethod(attrgetter("cita_cache"))
    def parse_cita(self, val: str) -> int:
        if isinstance(val, int):
            return val
        for cita in self.CITA:
            if val in cita.nm_cita:
                return cita.id_cita
        return 8

    @cachedmethod(attrgetter("hobby_cache"))
    def parse_hobby(self, val: str) -> int:
        if isinstance(val, int):
            return val
        for hobby in self.HOBBY:
            if val in hobby.nm_hobby:
                return hobby.id_hobby
        return 6

    @cachedmethod(attrgetter("pendaftaran_cache"))
    def parse_pendaftaran(self, val: str) -> int:
        if isinstance(val, int):
            return val
        for pendaftaran in self.PENDAFTARAN:
            if val in pendaftaran.nama:
                return pendaftaran.jenis_pendaftaran_id
        return 1

    def get_default(self) -> Dict[str, Any]:
        return {
            "nisn": self.nisn,
            "kebutuhan_khusus_id": self.kebutuhan_khusus_id,
            "lintang": self.lintang,
            "bujur": self.bujur,
            "jenis_tinggal_id": self.jenis_tinggal_id,
            "anak_keberapa": self.anak_keberapa,
            "alat_transportasi_id": self.alat_transportasi_id,
            "penerima_kps": self.penerima_kps,
            "penerima_kip": self.penerima_kip,
            "layak_pip": self.layak_pip,
            "id_layak_pip": self.id_layak_pip,
            "sekolah_id": self.sekolah.sekolah_id,
            "kewarganegaraan": self.kewarganegaraan,
            "no_kks": self.no_kks,
            "no_kps": self.no_kps,
            "no_kip": self.no_kip,
            "nm_kip": self.nm_kip,
            "kebutuhan_khusus_id_ayah": self.kebutuhan_khusus_id_ayah,
            "kebutuhan_khusus_id_ibu": self.kebutuhan_khusus_id_ibu,
            "nama_wali": self.nama_wali,
            "nik_wali": self.nik_wali,
            "tahun_lahir_wali": self.tahun_lahir_wali,
            "jenjang_pendidikan_wali": self.jenjang_pendidikan_wali,
            "pekerjaan_id_wali": self.pekerjaan_id_wali,
            "penghasilan_id_wali": self.penghasilan_id_wali,
            "nomor_telepon_rumah": self.nomor_telepon_rumah,
            "nomor_telepon_seluler": self.nomor_telepon_seluler,
            "email": self.email,
        }

    def get_default_registrasi(self) -> Dict[str, Any]:
        return {
            "registrasi_id": "Admin.model.RegistrasiPesertaDidik-3",
            # "peserta_didik_id":"",
            "jurusan_sp_id": "",
            # "sekolah_id":"",
            # "jenis_pendaftaran_id":1,
            # "tanggal_masuk_sekolah": "2021-07-05",
            "jenis_keluar_id": "",
            "tanggal_keluar": None,
            "keterangan": "",
            "no_skhun": "",
            # "nipd": "",
            # "id_hobby":6,
            # "id_cita":8,
            "no_seri_ijazah": "",
            "no_peserta_ujian": "",
            # "sekolah_asal": "",
            "a_pernah_paud": 0,
            "a_pernah_tk": 0,
            "jurusan_sp_id_str": "",
            "peserta_didik_id_str": "",
            "sekolah_id_str": "",
            "jenis_pendaftaran_id_str": "",
            "id_hobby_str": "",
            "id_cita_str": "",
            "jenis_keluar_id_str": "",
        }

    def get_default_longitudinal(self) -> Dict[str, Any]:
        return {
            "peserta_didik_longitudinal_id": "Admin.model.PesertaDidikLongitudinal-40",
            "semester_id": __semester__,
            # "peserta_didik_id": "",
            # "tinggi_badan": 75,
            # "berat_badan": 11,
            "jarak_rumah_ke_sekolah_km": 0,
            "jarak_rumah_ke_sekolah": 1,
            "waktu_tempuh_ke_sekolah": 0,
            # "menit_tempuh_ke_sekolah": 5,
            # "jumlah_saudara_kandung": 4,
            "vld_count": 0,
            "peserta_didik_longitudinal_id_str": "",
            "peserta_didik_id_str": "",
            "semester_id_str": "",
            # "lingkar_kepala": 20,
        }



import tkinter as tk
from dapodik import Dapodik
from typing import Optional

from . import LoginFrame


class MainApplication(tk.Frame):
    def __init__(self, master=None, cnf=None, **kw):
        super().__init__(master=master, cnf=cnf if cnf else {}, **kw)
        self.dapodik: Optional[Dapodik] = None
        self.login = LoginFrame(self, cnf=cnf)
        self.login.pack(side="right", fill="both", expand=True)

    def on_login(self):
        self.dapodik = self.login.dapodik()



import click
from random import randint
from typing import Callable, List

from dapodik import Dapodik, __semester__
from dapodik.peserta_didik import PesertaDidik, PesertaDidikLongitudinal
from dapodik.sekolah import Sekolah


class RandomLongitudinal:
    def __init__(
        self,
        dapodik: Dapodik,
        tinggi: Callable[[], int] = lambda: randint(125, 160),
        berat: Callable[[], int] = lambda: randint(20, 75),
        lingkar_kepala: Callable[[], int] = lambda: randint(20, 40),
        jarak_rumah: Callable[[], int] = lambda: randint(0, 20),
        jarak_waktu: int = 5,
    ):
        self.dapodik = dapodik
        self._tinggi = tinggi
        self._berat = berat
        self._lingkar_kepala = lingkar_kepala
        self._jarak_rumah = jarak_rumah
        self.jarak_waktu = jarak_waktu
        self.sekolah: Sekolah = self.dapodik.sekolah()
        if self.sekolah:
            click.echo(f"Berhasil login @{self.sekolah.nama}")
        self.PESERTA_DIDIK: List[PesertaDidik] = self.dapodik.peserta_didik(
            self.sekolah.sekolah_id,
            limit=1000000,
        )
        click.echo(f"Berhasil memuat peserta didik sebanyak {len(self.PESERTA_DIDIK)}")
        try:
            self.start()
        except Exception as e:
            click.echo(f"Terjadi error : {e}")
            raise e

    def tinggi(self) -> int:
        return self._tinggi()

    def berat(self) -> int:
        return self._berat()

    def lingkar_kepala(self) -> int:
        return self._lingkar_kepala()

    def jarak_rumah(self) -> int:
        return self._jarak_rumah()

    def get_new_longitudinal(
        self, peserta_didik: PesertaDidik
    ) -> PesertaDidikLongitudinal.Create:
        jarak_rumah = self.jarak_rumah()
        if jarak_rumah > 0:
            jarak_rumah_ke_sekolah_km = 0
            jarak_rumah_ke_sekolah = 2
        else:
            jarak_rumah_ke_sekolah_km = jarak_rumah
            jarak_rumah_ke_sekolah = 1
        jarak_waktu = jarak_rumah * self.jarak_waktu
        if peserta_didik.anak_keberapa > 1:
            saudara = peserta_didik.anak_keberapa - 1
        else:
            saudara = 0
        return PesertaDidikLongitudinal.Create(
            peserta_didik_id=peserta_didik.peserta_didik_id,
            tinggi_badan=self.tinggi(),
            berat_badan=self.berat(),
            jarak_rumah_ke_sekolah_km=jarak_rumah_ke_sekolah_km,
            jarak_rumah_ke_sekolah=jarak_rumah_ke_sekolah,
            waktu_tempuh_ke_sekolah=jarak_waktu // 60,
            menit_tempuh_ke_sekolah=jarak_waktu % 60,
            jumlah_saudara_kandung=saudara,
            lingkar_kepala=self.lingkar_kepala(),
        )

    def start(self):
        total = 0
        for peserta_didik in self.PESERTA_DIDIK:
            old_longitudinal: List[
                PesertaDidikLongitudinal
            ] = self.dapodik.peserta_didik_longitudinal(peserta_didik.peserta_didik_id)
            if self.check_longitudinals(old_longitudinal):
                click.echo(f"Lewati {peserta_didik} longitudinal sudah ada")
                continue
            longitudinal = self.get_new_longitudinal(peserta_didik)
            new_longitudinal = peserta_didik.create_longitudinal(longitudinal)
            click.echo(f"Longitudinal {peserta_didik} : {new_longitudinal}")
            total += 1
        if total > 0:
            click.echo(
                f"Berhasil membuat data longitudinal sebanyak {total} peserta didik"
            )
        else:
            click.echo(f"Semua peserta didik sudah memiliki data longitudinal")

    def check_longitudinals(
        self, longitudinals: List[PesertaDidikLongitudinal]
    ) -> bool:
        for longitudinal in longitudinals:
            if longitudinal.semester_id == __semester__:
                return True
        return False



from openpyxl.worksheet.worksheet import Worksheet
from typing import Any, Dict, List, Optional

from dapodik import Dapodik, __semester__
from dapodik.peserta_didik import PesertaDidik, PesertaDidikLongitudinal

from . import ALL_DATA_INDIVIDU, DATA_LONGITUDINAL
from dapodix.utils import get_workbook, snake_to_title


class EksporPesertaDidikCommand:
    MAPPING_INDIVIDU: Dict[str, str] = ALL_DATA_INDIVIDU
    MAPPING_LONGITUDINAL: Dict[str, str] = DATA_LONGITUDINAL

    def __init__(
        self,
        dapodik: Dapodik,
        filepath: str,
        sheet: str = "Peserta Didik",
        header: bool = True,
        skip_longitudinal: bool = False,
    ):
        self.dapodik = dapodik
        self.offset = 2 if header else 1
        self.skip_longitudinal = skip_longitudinal
        self.sekolah = self.dapodik.sekolah()
        self.peserta_didik = self.get_peserta_didik()
        self.WORKBOOK = get_workbook(filepath)
        self.WORKSHEET: Worksheet = self.WORKBOOK.active
        self.WORKSHEET.title = sheet
        if header:
            self.add_header()
        for index, peserta_didik in enumerate(self.peserta_didik):
            self.peserta_didik_to_row(peserta_didik, index + self.offset)
        if not filepath.endswith(".xlsx"):
            filepath += ".xlsx"
        self.WORKBOOK.save(filename=filepath)

    def get_peserta_didik(self) -> List[PesertaDidik]:
        return self.dapodik.peserta_didik(sekolah_id=self.sekolah.sekolah_id)

    def add_header(self, row: int = 1):
        for name, col in self.MAPPING_INDIVIDU.items():
            self.WORKSHEET[f"{col}{row}"] = snake_to_title(name)
        for name, col in self.MAPPING_LONGITUDINAL.items():
            self.WORKSHEET[f"{col}{row}"] = snake_to_title(name)

    def peserta_didik_to_row(self, pd: PesertaDidik, row: int):
        for name, col in self.MAPPING_INDIVIDU.items():
            if not hasattr(pd, name):
                continue
            value = getattr(pd, name)
            if value is None:
                continue
            self.WORKSHEET[f"{col}{row}"] = value
        if not self.skip_longitudinal:
            for name, col in self.MAPPING_LONGITUDINAL.items():
                longitudinal: Optional[PesertaDidikLongitudinal] = None
                longitudinals = self.dapodik.peserta_didik_longitudinal(
                    peserta_didik_id=pd.peserta_didik_id,
                )
                for ltd in longitudinals:
                    if ltd.semester_id == __semester__:
                        longitudinal = ltd
                        break
                if not hasattr(longitudinal, name):
                    continue
                value = getattr(longitudinal, name)
                if value is None:
                    continue
                self.WORKSHEET[f"{col}{row}"] = value



import os
from datetime import date
from openpyxl import load_workbook, Workbook
from openpyxl.worksheet.worksheet import Worksheet
from typing import Any, Dict, List


def get_workbook(filepath: str) -> Workbook:
    """get_workbook Open Workbook if file exist

    Args:
        filepath (str): File path

    Returns:
        Workbook: Workbook
    """
    if os.path.isfile(filepath):
        return load_workbook(filepath)
    return Workbook()


def get_data_ws(ws: Worksheet, mapping: Dict[str, str], row: int) -> Dict[str, Any]:
    """get_data_ws get data from worksheet

    Args:
        ws (Worksheet): Openpyxl worksheet
        mapping (Dict[str, str]): Key and collumn combination
        row (int): Row number

    Returns:
        Dict[str, Any]: Data?
    """
    result: Dict[str, Any] = dict()
    for key, col in mapping.items():
        result[key] = ws[f"{col}{row}"].value
    return result


def get_data_excel(
    filepath: str,
    sheet: str,
    rows: List[int],
    mapping: Dict[str, str],
) -> Dict[int, Dict[str, Any]]:
    """get_data_excel get data from excel file

    Args:
        filepath (str): File directory
        sheet (str): Worksheet name
        rows (List[int]): Number of row
        mapping (Dict[str, str]): Key and collumn combination

    Returns:
        Dict[int, Dict[str, Any]]: Data?
    """
    results: Dict[int, Dict[str, Any]] = dict()
    wb = load_workbook(filepath)
    ws = wb[sheet]
    for row in rows:
        results[row] = get_data_ws(ws, mapping, row)
    return results


def parse_range(value: str) -> List[int]:
    results: List[int] = list()
    if value.isdigit():
        results.append(int(value))
    elif "," in value:
        for val in value.split(","):
            # For 1,2
            results.extend(parse_range(val))
    elif "-" in value:
        start, end = value.split("-")
        if start.isdigit() and end.isdigit():
            if start == end:
                # For 1-1
                results.append(int(start))
            else:
                # For 1-2
                results.extend(range(int(start), int(end) + 1))
    return results


def snake_to_title(text: str) -> str:
    text = text.replace("_", " ")
    return text.title()


def parse_date(text: str, SEPARATOR: str = "") -> date:
    if SEPARATOR:
        pass
    elif "/" in text:
        SEPARATOR = "/"
    elif ":" in text:
        SEPARATOR = ":"
    else:
        SEPARATOR = " "
    tanggal, bulan, tahun = text.split(SEPARATOR)
    return date(int(tahun), int(bulan), int(tanggal))



DATA_INDIVIDU = {
    "nisn": "B",
    "nama": "C",
    "jenis_kelamin": "D",
    "nik": "E",
    "no_kk": "F",
    "tempat_lahir": "G",
    "tanggal_lahir": "H",
    "reg_akta_lahir": "I",
    "agama_id": "J",
    "alamat_jalan": "K",
    "rt": "L",
    "rw": "M",
    "nama_dusun": "N",
    "kode_wilayah": "O",
    "desa_kelurahan": "P",
    "kode_pos": "Q",
    "anak_keberapa": "R",
    "nomor_telepon_seluler": "AE",
    "email": "AF",
}

DATA_REGISTRASI = {
    "nipd": "A",
    "jenis_pendaftaran_id": "AG",
    "tanggal_masuk_sekolah": "AH",
    "sekolah_asal": "AI",
    "id_hobby": "AJ",
    "id_cita": "AK",
    "a_pernah_paud": "AL",
    "a_pernah_tk": "AM",
}

DATA_LONGITUDINAL = {
    "tinggi_badan": "AN",
    "berat_badan": "AO",
    "lingkar_kepala": "AP",
    "jarak_rumah_ke_sekolah": "AQ",
    "menit_tempuh_ke_sekolah": "AR",
    "jumlah_saudara_kandung": "AS",
}

DATA_AYAH = {
    "nama_ayah": "S",
    "nik_ayah": "T",
    "tahun_lahir_ayah": "U",
    "jenjang_pendidikan_ayah": "V",
    "pekerjaan_id_ayah": "W",
    "penghasilan_id_ayah": "X",
}

DATA_IBU = {
    "nama_ibu_kandung": "Y",
    "nik_ibu": "Z",
    "tahun_lahir_ibu": "AA",
    "jenjang_pendidikan_ibu": "AB",
    "pekerjaan_id_ibu": "AC",
    "penghasilan_id_ibu": "AD",
}

ALL_DATA_INDIVIDU = dict(DATA_INDIVIDU)
ALL_DATA_INDIVIDU.update(DATA_AYAH)
ALL_DATA_INDIVIDU.update(DATA_IBU)
ALL_DATA_INDIVIDU.update(DATA_REGISTRASI)



import attr
from click import Context
from typing import Optional

from dapodik import Dapodik, __semester__


@attr.dataclass
class ContextObject:
    username: str = ""
    password: str = ""
    pengguna: int = 0
    server: str = "http://localhost:5774/"
    semester: str = __semester__
    rememberme: bool = True
    _dapodik: Optional[Dapodik] = None

    @property
    def dapodik(self) -> Dapodik:
        if self._dapodik:
            return self._dapodik
        assert self.username
        assert self.password
        self._dapodik = Dapodik(
            username=self.username,
            password=self.password,
            semester_id=self.semester,
            server=self.server,
            pengguna=self.pengguna,
            rememberme=self.rememberme,
        )
        return self._dapodik


class ClickContext(Context):
    obj: ContextObject



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dapodix', 'dapodix.gui', 'dapodix.peserta_didik']

package_data = \
{'': ['*']}

install_requires = \
['cachetools>=4.2.2,<5.0.0',
 'click>=8.0.1,<9.0.0',
 'dapodik>=0.16.6,<0.17.0',
 'openpyxl>=3.0.7,<4.0.0']

entry_points = \
{'console_scripts': ['dapodix = dapodix.__main__:main']}

setup_kwargs = {
    'name': 'dapodix',
    'version': '0.4.0',
    'description': 'Alat bantu aplikasi Dapodik kemdikbud',
    'long_description': '# dapodix\n\n[![dapodix - PyPi](https://img.shields.io/pypi/v/dapodix)](https://pypi.org/project/dapodix/)\n[![Download](https://img.shields.io/badge/Download-Unduh-brightgreen)](https://github.com/hexatester/dapodix/archive/master.zip)\n[![Donate DANA](https://img.shields.io/badge/Donasi-Saweria-blue)](https://saweria.co/hexatester)\n[![Tutorial](https://img.shields.io/badge/Tutorial-Penggunaan-informational)](https://github.com/hexatester/dapodix/wiki)\n[![Group Telegram](https://img.shields.io/badge/Telegram-Group-blue.svg)](https://t.me/dapodik_2021)\n[![codecov](https://codecov.io/gh/hexatester/dapodix/branch/master/graph/badge.svg)](https://codecov.io/gh/hexatester/dapodix)\n[![LISENSI](https://img.shields.io/github/license/hexatester/dapodix)](https://github.com/hexatester/dapodix/blob/master/LISENSI)\n\nAlat bantu aplikasi dapodik.\n\n## Install\n\nPastikan minimal [python versi 3.7](https://www.python.org/downloads/release/python-3711/) terinstall,\nkemudian jalankan perintah di bawah dalam Command Prompt atau Powershell (di Windows + X):\n\n```bash\npip install --upgrade dapodix\n```\n\n## Fitur yang akan datang\n\n- Download data ke Excel\n- Petunjuk validasi\n- Bulk update dari Excel\n\n## Legal / Hukum\n\nKode ini sama sekali tidak berafiliasi dengan, diizinkan, dipelihara, disponsori atau didukung oleh [Kemdikbud](https://kemdikbud.go.id/) atau afiliasi atau anak organisasinya. Ini adalah perangkat lunak yang independen dan tidak resmi. _Gunakan dengan risiko Anda sendiri._\n',
    'author': 'hexatester',
    'author_email': 'hexatester@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hexatester/dapodix',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from .version import __version__  # NOQA
from .click_context import ContextObject, ClickContext
from .peserta_didik import RegistrasiPesertaDidikCommand

__all__ = ["ClickContext", "ContextObject", "RegistrasiPesertaDidikCommand"]



import click
from typing import List

from dapodik import Dapodik
from dapodik.sekolah import Sekolah
from dapodik.peserta_didik import PesertaDidik, RegistrasiPesertaDidik

from dapodix.utils import parse_date


class TanggalMasukCommand:
    def __init__(self, dapodik: Dapodik, tanggal: str):
        self.dapodik = dapodik
        self.tanggal = parse_date(tanggal)
        self.sekolah = self.dapodik.sekolah()
        self.peserta_didik: List[PesertaDidik] = self.dapodik.peserta_didik(
            sekolah_id=self.sekolah.sekolah_id
        )

    def update_registrasi(self):
        for pd in self.peserta_didik:
            if not pd.tanggal_masuk_sekolah:
                click.echo(f"Lewati {pd} tidak memiliki tanggal masuk sekolah")
                continue
